@@ -2,43 +2,16 @@
 # -*- coding: utf-8 -*-
 
 from random import shuffle
-
-try:
-	import cPickle as pickle
-except ImportError:
-	import pickle
-
-import os, sys, re
-
-from grab.error import GrabError
-
+import os, sys, re, multiprocessing, argparse, sqlite3, atexit
 from time import time, sleep
 
-import multiprocessing
-
 from mako.lookup import TemplateLookup
+from bottle import route, run, static_file, default_app, request
 
-from bottle import route, run, static_file, default_app, redirect, request
-
-from Daemo import Daemon
-
-import argparse
-
-import sqlite3
-
-import atexit
+from Daemo import Daemon, DaemonError
 
 from TechParser import get_conf, recommend, parser
-
-try:
-	from urllib.parse import urlencode
-except ImportError:
-	from urllib import urlencode
-
-try:
-	unicode_ = unicode
-except NameError:
-	unicode_ = str
+from TechParser.py2x import unicode_, range, pickle, urlencode
 
 if get_conf.config is None:
 	get_conf.set_config_auto()
@@ -109,6 +82,10 @@ def log(text, f=sys.stdout, add_newline=True, clear_str=False,
 	f.write(text)
 	f.flush()
 
+def logerr(*args, **kwargs):
+	kwargs['f'] = sys.stderr
+	log(*args, **kwargs)
+
 def split_into_pages(articles, n=30):
 	"""Split list into pages"""
 	
@@ -124,7 +101,7 @@ def split_into_pages(articles, n=30):
 		else:
 			pages[-1].append(j)
 		
-		i+=1
+		i += 1
 	
 	return pages
 
@@ -137,14 +114,14 @@ def simple_plural(n, s):
 
 def show_progress(s, shared_object):
 	progress = round(shared_object.value, 2)
-	return "\033[0;32m[{}%]\033[0m ".format(progress)+s
+	return "\033[0;32m[{0}%]\033[0m ".format(progress)+s
 
 def parse_site(queue, articles, progress):
 	config = get_conf.config
 	
 	while not queue.empty():
 		site = queue.get()
-		s = "Got {} {} from {}"
+		s = "Got {0} {1} from {2}"
 		
 		if site not in config.sites_to_parse:
 			d = config.rss_feeds[site]
@@ -153,7 +130,7 @@ def parse_site(queue, articles, progress):
 		
 		update_progress(progress,
 			100.0 / (len(config.sites_to_parse) + len(config.rss_feeds)) / 2.0)
-		log(show_progress("Parsing articles from {}".format(site), progress))
+		log(show_progress("Parsing articles from {0}".format(site), progress))
 		
 		if 'module' not in d:
 			url = d.get('url', 'about:blank')
@@ -174,8 +151,8 @@ def parse_site(queue, articles, progress):
 				log(show_progress(s.format(len(new_articles),
 					simple_plural(len(new_articles), 'article'), site), progress))
 			except Exception as error:
-				log('Fail')
-				log(str(error), f=sys.stderr)
+				logerr('Fail')
+				logerr(str(error))
 		else:
 			module = d["module"]
 			kwargs = d["kwargs"]
@@ -189,8 +166,8 @@ def parse_site(queue, articles, progress):
 				log(show_progress(s.format(len(found),
 					simple_plural(len(found), 'article'), site), progress))
 			except Exception as error:
-				log("Failed to parse articles from {}".format(site))
-				log(str(error), f=sys.stderr)
+				logerr("Failed to parse articles from {0}".format(site))
+				logerr(str(error))
 
 def update_progress(shared_object, num):
 	shared_object.value += num
@@ -212,8 +189,6 @@ def dump_articles(filename="articles_dumped"):
 	for i in get_conf.config.rss_feeds:
 		main_queue.put(i)
 	
-	s = "\033[0;32m[{}%]\033[0m Parsing articles from {}... "
-	
 	pool = multiprocessing.Pool(processes=get_conf.config.num_threads)
 	
 	for i in range(get_conf.config.num_threads):
@@ -264,7 +239,7 @@ def dump_articles(filename="articles_dumped"):
 		shuffle(list_articles)
 		list_articles = [[a, -1] for a in list_articles]
 	
-	log("Dumping data to file: {}...".format(filename))
+	log("Dumping data to file: {0}...".format(filename))
 	
 	dumped = pickle.dumps(list_articles)
 	path = os.path.join(os.path.expanduser("~"), ".tech-parser")
@@ -324,11 +299,11 @@ def filter_articles(articles):
 def load_articles(filename="articles_dumped"):
 	"""Load articles from ~/.tech-parser/<filename>"""
 	
-	log("Reading articles from file: {}...".format(filename))
+	log("Reading articles from file: {0}...".format(filename))
 	try:
 		f = open(os.path.join(logdir, filename), 'rb')
 	except IOError:
-		log("File '{}' doesn't exist: returning empty list".format(filename))
+		log("File '{0}' doesn't exist: returning empty list".format(filename))
 		return []
 	
 	dumped = f.read()
@@ -346,48 +321,46 @@ def serve_static(filename):
 	
 	return static_file(filename, root=static_dir_path)
 
-@route('/go/<addr:path>')
-def go_to_url(addr):
-	recommend.add_article(addr, db=get_conf.config.db)
-	redirect(addr)
+def update_liked_disliked():
+	global liked, disliked, liked_links, disliked_links
+
+	config = get_conf.config
+	
+	liked = recommend.get_interesting_articles(db=config.db)
+	disliked = recommend.get_blacklist(db=config.db)
+	liked_links = [i['link'] for i in liked]
+	disliked_links = [i['link'] for i in disliked]
 
 @route('/histadd/<addr:path>')
 def add_to_history(addr):
-	global liked, disliked, liked_links, disliked_links
 	
-	config = get_conf.config
+	recommend.add_article(addr, db=get_conf.config.db)
+	
+	update_liked_disliked()
 	
-	recommend.add_article(addr, db=config.db)
-	liked = recommend.get_interesting_articles(db=config.db)
-	disliked = recommend.get_blacklist(db=config.db)
-	liked_links = [i['link'] for i in liked]
-	disliked_links = [i['link'] for i in disliked]
-
 @route('/blacklistadd/<addr:path>')
 def add_to_blacklist(addr):
-	global liked, disliked, liked_links, disliked_links
+	recommend.add_article_to_blacklist(addr, db=get_conf.config.db)
 	
-	config = get_conf.config
-	
-	recommend.add_article_to_blacklist(addr, db=config.db)
-	liked = recommend.get_interesting_articles(db=config.db)
-	disliked = recommend.get_blacklist(db=config.db)
-	liked_links = [i['link'] for i in liked]
-	disliked_links = [i['link'] for i in disliked]
+	update_liked_disliked()
 
 @route('/blacklistrm/<addr:path>')
 def rm_from_blacklist(addr):
 	recommend.remove_from_blacklist(addr, db=get_conf.config.db)
+	
+	update_liked_disliked()
 
 @route('/histrm/<addr:path>')
 def rm_from_history(addr):
 	recommend.remove_from_history(addr, db=get_conf.config.db)
+	
+	update_liked_disliked()
 
 @route('/history')
 @route('/history/<page_number>')
 def show_history(page_number=1):
 	history_page = mylookup.get_template('history.html')
-	q = request.GET.get('q', '')
+	q = unicode_(request.GET.get('q', ''))
 	
 	articles = recommend.get_interesting_articles(db=get_conf.config.db)
 	
@@ -413,13 +386,13 @@ def show_history(page_number=1):
 		num_pages=len(articles),
 		page_num=page_number,
 		q=q,
-		all_articles=all_articles,)
+		all_articles=all_articles)
 
 @route('/blacklist')
 @route('/blacklist/<page_number>')
 def show_blacklist(page_number=1):
 	history_page = mylookup.get_template('blacklist.html')
-	q = request.GET.get('q', '')
+	q = unicode_(request.GET.get('q', ''))
 	
 	articles = recommend.get_blacklist(db=get_conf.config.db)
 	
@@ -448,12 +421,13 @@ def show_blacklist(page_number=1):
 		all_articles=all_articles)
 
 def has_words(qs, article):
-	"""Check if article title contains words:"""
+	"""Check if article contains words"""
 	
 	title = article['title'].lower()
+	summary = article['summary'].lower()
 	
 	for i in qs:
-		if i not in title:
+		if i not in title and i not in summary:
 			return False
 	return True
 
@@ -485,7 +459,7 @@ def article_list(page_number=1):
 	"""Show list of articles | Search for articles"""
 	
 	main_page = mylookup.get_template("articles.html")
-	q = request.GET.get('q', '')
+	q = unicode_(request.GET.get('q', ''))
 	
 	try:
 		page_number = int(page_number)
@@ -531,25 +505,21 @@ class ParserDaemon(Daemon):
 		super(ParserDaemon, self).__init__(pidfile, stdout=so, stderr=se)
 	
 	def onStart(self):
-		config = get_conf.config
-		
-		p1 = multiprocessing.Process(target=dump_articles_per, args=(config.update_interval,))
-		atexit.register(p1.terminate)
-		p1.start()
-		run(host=config.host, port=config.port, server=config.server)
+		run_server(host=get_conf.config.host, port=get_conf.config.port)
 
 def is_hostname(hostname):
 	if len(hostname) > 255:
 		return False
 	if hostname[-1] == ".":
 		hostname = hostname[:-1]
-	allowed = re.compile("(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
+	allowed = re.compile(r"(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
 	return all(allowed.match(x) for x in hostname.split("."))
 
 def run_server(host, port):
 	config = get_conf.config
 	
-	p1 = multiprocessing.Process(target=dump_articles_per, args=(config.update_interval,))
+	p1 = multiprocessing.Process(target=dump_articles_per,
+		args=(config.update_interval,))
 	atexit.register(p1.terminate)
 	p1.start()
 	run(host=host, port=port, server=config.server)
@@ -588,10 +558,10 @@ Available commands: start|stop|restart|update|run HOST:PORT""")
 				is_port_correct = False
 				
 			if not is_host_correct:
-				log("Incorrect hostname", f=sys.stderr)
+				logerr("Invalid hostname")
 				sys.exit(1)
 			elif not is_port_correct:
-				log("Incorrect port", f=sys.stderr)
+				logerr("Invalid port")
 				sys.exit(1)
 			else:
 				run_server(host, port)
@@ -600,10 +570,18 @@ Available commands: start|stop|restart|update|run HOST:PORT""")
 		elif args.action[0] == "start":
 			running_as_daemon = True
 			parser_daemon = ParserDaemon()
-			parser_daemon.start()
+			try:
+				parser_daemon.start()
+			except DaemonError as e:
+				logerr('Failed to start server: {0}'.format(e))
+				sys.exit(1)
 		elif args.action[0] == "stop":
 			running_as_daemon = True
 			parser_daemon = ParserDaemon()
-			parser_daemon.stop()
+			try:
+				parser_daemon.stop()
+			except DaemonError as e:
+				logerr('Failed to stop server: {0}'.format(e))
+				sys.exit(1)
 else:
 	app = default_app()
